var Search = function()
{
	var search = {};
	var debug = 0;
	
	var bugIcons = [ 'bug-yellow.png', 'bug-red.png', 'bomb.png' ];
	
	var alignments;
	var firstLoad = true;
	
	var itemByName = {}; // To keep track of which QiaCubes and LibPlates we have already seen
	var alignmentByName = {}; // Maps alignment name -> alignment object
	var alignmentsByPat = {}; // Maps patient name -> [] of alignments for that patient
	
	// Page initialization
	search.initPage = function()
	{
		// Step 1
		Events.addEventHandler('noFilter', 'change', search.filterOnChange);
		Events.addEventHandler('highHetFilter', 'change', search.filterOnChange);
		Events.addEventHandler('libPlateFilter', 'change', search.filterOnChange);
		Events.addEventHandler('qiaCubeFilter', 'change', search.filterOnChange);
		Events.addEventHandler('recentFilters', 'change', search.recentFilterOnChange);
		
		// Navigation
		/*
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', search.submit);
		*/

		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=GetFlaggedAlignments';
		Wizard.showLoadingAnimation('Loading flagged alignments...');
		Wizard.asyncJsonRequest(url, search.onAlignmentsLoaded);
	}
	
	search.onAlignmentsLoaded = function(response)
	{
		alignments = response.alignments;		
		if (alignments.length == 0)
		{
			Wizard.setFatalError('No flagged alignments available for processing.');
			return;
		}
		if (firstLoad) 
		{
			search.initFilterLists();
		}
		else 
		{
			search.updateFilterLists();
		}
		firstLoad = false;
		search.primaryAnalysis();

		search.updateTable();
	}
	
	/**
		Initialize the QiaCube and LibPlate filter lists.
		This is done on the first load only.
	*/
	search.initFilterLists = function()
	{
		var allQiaCubes = [];
		var allLibPlates = [];
		
		for (var itemNo = 0; itemNo < alignments.length; itemNo++)
		{
			var aligned = alignments[itemNo];
			var rna = aligned.rna;
			var qiaCubeRun = rna.QIACUBE_DATE + '#' + rna.QIACUBE_RUN_NO;
			var qiaCubeRunPos = qiaCubeRun + '@' + rna.QIACUBE_POSITION;
	
			var qiaCube = itemByName[qiaCubeRun];
			if (!qiaCube)
			{
				qiaCube = new QiaCube(rna.QIACUBE_DATE, rna.QIACUBE_RUN_NO);
				qiaCube.posCount = 1;
				allQiaCubes[allQiaCubes.length] = qiaCube;
				itemByName[qiaCubeRun] = qiaCube;
			}
			else
			{
				if (!itemByName[qiaCubeRunPos])
				{
					qiaCube.posCount++;
					itemByName[qiaCubeRunPos] = qiaCube;
				}
			}
			
			var lib = aligned.lib;
			var libPlate = lib.bioWell.bioPlate;
	
			var plate = itemByName[libPlate.name];
			if (!plate)
			{
				plate = new LibPlate(libPlate.name);
				plate.posCount = 1;
				allLibPlates[allLibPlates.length] = plate;
				itemByName[libPlate.name] = plate;
			}
			else
			{
				var platePos = libPlate.name+'@'+lib.bioWell.location;
				if (!itemByName[platePos])
				{
					plate.posCount++;
					itemByName[platePos] = plate;
				}
			}
		}
		
		// Sort in logical order
		allQiaCubes.sort(Sort.sortByName);
		allLibPlates.sort(Sort.sortByName);
		
		var frm = document.forms['reggie'];
		for (var plateNo = 0; plateNo < allLibPlates.length; plateNo++)
		{
			var libPlate = allLibPlates[plateNo];
			var option = new Option(libPlate.name + " [" + libPlate.posCount + "]", libPlate.name);
			option.libPlate = libPlate;
			frm.libPlateFilter[frm.libPlateFilter.length] = option;
		}
		for (var qiaCubeNo = 0; qiaCubeNo < allQiaCubes.length; qiaCubeNo++)
		{
			var qiaCube = allQiaCubes[qiaCubeNo];
			var option = new Option(qiaCube.title + " [" + qiaCube.posCount + "]", qiaCube.name);
			option.qiaCube = qiaCube;
			frm.qiaCubeFilter[frm.qiaCubeFilter.length] = option;
		}
	}
	
	
	/**
		Update the QiaCube and LibPlate filter lists with new options.
		This is done after the first load. The options are added to the end of the lists.
	*/
	search.updateFilterLists = function()
	{
		var frm = document.forms['reggie'];
		
		for (var itemNo = 0; itemNo < alignments.length; itemNo++)
		{
			var aligned = alignments[itemNo];
			var rna = aligned.rna;
			var qiaCubeRun = rna.QIACUBE_DATE + '#' + rna.QIACUBE_RUN_NO;
			var qiaCubeRunPos = qiaCubeRun + '@' + rna.QIACUBE_POSITION;
	
			var qiaCube = itemByName[qiaCubeRun];
			if (!qiaCube)
			{
				qiaCube = new QiaCube(rna.QIACUBE_DATE, rna.QIACUBE_RUN_NO);
				itemByName[qiaCubeRun] = qiaCube;
				var option = new Option(qiaCube.title, qiaCube.name);
				option.qiaCube = qiaCube;
				frm.qiaCubeFilter[frm.qiaCubeFilter.length] = option;
			}
			
			var lib = aligned.lib;
			var libPlate = lib.bioWell.bioPlate;
	
			var plate = itemByName[libPlate.name];
			if (!plate)
			{
				plate = new LibPlate(libPlate.name);
				itemByName[libPlate.name] = plate;
				
				var option = new Option(libPlate.name, libPlate.name);
				option.libPlate = libPlate;
				frm.libPlateFilter[frm.libPlateFilter.length] = option;
			}
		}
	}
	
	/**
		Callback when some of the filter options has changed.
		This method will detect which one was modified, and
		reset all other filters. Then, it will issue a 
		request to the server to load alignments related to the
		selected filter.
	*/
	search.filterOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		
		var noFilter = Doc.element('noFilter');
		var highHet = Doc.element('highHetFilter');
		var libPlate = Doc.element('libPlateFilter');
		var qiaCube = Doc.element('qiaCubeFilter');
				
		if (event.target != noFilter) noFilter.checked = false;
		if (event.target != highHet) highHet.checked = false;
		if (event.target != libPlate) libPlate.selectedIndex = 0;
		if (event.target != qiaCube) qiaCube.selectedIndex = 0;
		
		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=GetFlaggedAlignments';
		
		var msg = 'Loading flagged alignments...';
		var hasFilter = false;
		var newRecentFilter = null;
		if (libPlate.selectedIndex > 0)
		{
			url += '&libPlateFilter='+encodeURIComponent(libPlate.value);
			msg = 'Loading alignments related to ' + libPlate.value;
			hasFilter = true;
			newRecentFilter = { 'value': 'lp:'+libPlate.value, 'text': libPlate.value };
		}
		else if (qiaCube.selectedIndex > 0)
		{
			var qiaCube = frm.qiaCubeFilter[frm.qiaCubeFilter.selectedIndex].qiaCube;
			url += '&qiaCubeFilter='+encodeURIComponent(qiaCube.name);
			msg = 'Loading alignments related to QiaCube ' + qiaCube.title;
			hasFilter = true;
			newRecentFilter = { 'value': 'qc:'+qiaCube.name, 'text': qiaCube.title };
		}
		else if (highHet.checked)
		{
			url += '&highHetFilter=1';
			hasFilter = true;
		}
		else
		{
			noFilter.checked = true;
		}
		
		Wizard.showLoadingAnimation(msg);
		Wizard.asyncJsonRequest(url, search.onAlignmentsLoaded);
		
		if (newRecentFilter != null)
		{
			var recentFilter = Doc.element('recentFilters');
			for (var i = 0; i < recentFilter.length; i++)
			{
				if (recentFilter[i].value == newRecentFilter.value)
				{
					recentFilter.remove(i);
					break;
				}
			}
			recentFilter.add(new Option(newRecentFilter.text, newRecentFilter.value), 1);
			Doc.show(recentFilter, 'inline');
		}
		
		if (hasFilter)
		{
			search.updateTable(true);
		}
		else
		{
			Doc.element('itemTable').innerHTML = '';
		}
	}
	

	search.recentFilterOnChange = function(event)
	{
		var list = event.currentTarget;
		var value = list.value;
		var prefix = value.substr(0, 3);
		list.selectedIndex = 0;
		if (prefix == 'qc:')
		{
			Data.set(event.currentTarget, 'qiacuberun', value.substr(3));
			search.setQiaCubeFilter(event);
		}
		else if (prefix == 'lp:')
		{
			Data.set(event.currentTarget, 'libplate', value.substr(3));
			search.setLibPlateFilter(event);
		}
		
	}
	
	/**
		Event handler for clicking on a QiaCube run, which should
		enable the filter for that run and re-load alignments 
		related to it. Updating the selected item in the filter list
		and sending a 'change' event should do the trick.
	*/
	search.setQiaCubeFilter = function(event)
	{
		var qiaCubeRun = Data.get(event.currentTarget, 'qiacuberun');
		var frm = document.forms['reggie'];
		Forms.selectListOption(frm.qiaCubeFilter, qiaCubeRun);
		Events.sendChangeEvent(frm.qiaCubeFilter);
	}
	
	/**
		Event handler for clicking on a LibPlate, which should
		enable the filter for that run and re-load alignments 
		related to it. Updating the selected item in the filter list
		and sending a 'change' event should do the trick.
	*/
	search.setLibPlateFilter = function(event)
	{
		var libplate = Data.get(event.currentTarget, 'libplate');
		var frm = document.forms['reggie'];
		Forms.selectListOption(frm.libPlateFilter, libplate);
		Events.sendChangeEvent(frm.libPlateFilter);
	}

	
	search.updateTable = function(noContextAnalysis)
	{
		var frm = document.forms['reggie'];
		
		var stepTitle = 'Flagged alignments';
		var filter = null;
		var positionProperty = null;
		var highHetFilter = false;
		var qiacubefilter = false;
		var libplatefilter = false;
		var qiaCube = null;
		var libPlate = null;
		
		if (Doc.element('highHetFilter').checked)
		{
			alignments.sort(Sort.sortAlignmentsByPatientName);
			highHetFilter = true;
			filter = function(alignment)
			{
				return alignment.QC_GENOTYPE_HET_PCT > 65;
			}
		}
		else if (frm.qiaCubeFilter.selectedIndex > 0)
		{
			alignments.sort(Sort.sortAlignmentsByQiaCubePosition);
			qiaCube = frm.qiaCubeFilter[frm.qiaCubeFilter.selectedIndex].qiaCube;
			qiacubefilter = qiaCube.name;
			filter = Filter.qiaCubeFilter(qiaCube.name);
			positionProperty = Property.QiaCubePosition;
			stepTitle = 'Alignments related to QiaCube ' + qiaCube.title;
		}
		else if (frm.libPlateFilter.selectedIndex > 0)
		{
			alignments.sort(Sort.sortAlignmentsByLibPlatePosition);
			libPlate = frm.libPlateFilter[frm.libPlateFilter.selectedIndex].libPlate;
			libplatefilter = libPlate.name;
			filter = Filter.libPlateFilter(libPlate.name);
			positionProperty = Property.LibPlatePosition;
			stepTitle = 'Alignments related to ' + libPlate.name;
		}
		else
		{
			// No filter
			alignments.sort(Sort.sortAlignmentsByQiaCubePosition);
		}
		
		if (filter && positionProperty && !noContextAnalysis)
		{
			search.contextAnalysis(filter, positionProperty);
		}

		var filtered = [];
		var tbody = [];
		if (filter)
		{
			var lastPos = null;
			for (var itemNo = 0; itemNo < alignments.length; itemNo++)
			{
				var aligned = alignments[itemNo];
				if (filter(aligned)) 
				{
					filtered[filtered.length] = aligned;
					var tb = { 'start': 1, 'end': 1 };	// The default is to start a new <tbody> section for each alignment
					tbody[tbody.length] = tb;
					if (positionProperty)
					{
						var pos = positionProperty(aligned);
						if (pos == lastPos)
						{
							tbody[tbody.length-2].end = 0;
							tb.start = 0;
						}
						lastPos = pos;
					}
				}
			}
		}
		else
		{
			filtered = alignments;
		}
		
		
		
		var html = '';
		html += '<thead class="bg-filled-100">';
		html += '<tr>';
		html += '<th>Patient</th>';
		html += '<th>QiaCube</th>';
		html += '<th>LibPlate</th>';
		html += '<th>HET</th>';
		html += '<th>Alignment</th>';
		html += '<th class="dottedleft" colspan="2">Warnings and messages</th>';
		html += '<th class="dottedleft">Alignment</th>';
		html += '<th>HET</th>';
		html += '<th>QiaCube</th>';
		html += '<th>LibPlate</th>';
		html += '<th>Patient</th>';
		html += '</tr>';
		html += '</thead>';
		
		var tbodyHtml = '';
		var numListed = 0;
		var tbodySpan = 0;
		for (var itemNo = 0; itemNo < filtered.length; itemNo++)
		{
			var aligned = filtered[itemNo];
			var cmp = aligned.compare;
			var lib = aligned.lib;
			var rna = aligned.rna;
			var libPlate = lib.bioWell.bioPlate;
			numListed++;

			var tb = tbody[itemNo] || { 'start': 1, 'end': 1 };
			var rowspan = cmp.messages.length || 1;
			tbodySpan += rowspan;
			
			if (!tb.start)
			{
				tbodyHtml += '<tr class="dotted-above">';
			}
			
			var tmp = search.getAlignmentHtml(aligned);
			
			if (!libplatefilter)
			{
				tbodyHtml += '<td rowspan="'+rowspan+'">'+HTMLHelper.libPlateLink(libPlate.name, lib.bioWell.location, libplatefilter, false)+'</td>';
			}
			tbodyHtml += '<td rowspan="'+rowspan+'">'+Reggie.formatNumber(aligned.QC_GENOTYPE_HET_PCT, '%', -1)+'</td>';
			tbodyHtml += '<td rowspan="'+rowspan+'" class="text-col">'+tmp.name+tmp.icons+'</td>';
			tbodyHtml += '<td rowspan="'+rowspan+'" class="dottedleft bg-messages">'+tmp.tooltip+'</td>';
			// The first message is on the main <tr>
			if (cmp.messages.length > 0)
			{
				var msg = cmp.messages[0];
				tbodyHtml += search.getMessageHtml(msg, aligned, alignmentByName, qiacubefilter, libplatefilter);
			}
			else
			{
				tbodyHtml += '<td class="bg-messages"></td><td class="dottedleft" colspan="5"></td>';
			}
			tbodyHtml += '</tr>';
			
			// If there are more messages, add more <tr>
			for (var mNo = 1; mNo < cmp.messages.length; mNo++)
			{
				var msg = cmp.messages[mNo];
				tbodyHtml += '<tr>'+search.getMessageHtml(msg, aligned, alignmentByName, qiacubefilter, libplatefilter) + '</tr>';
			}
			
			if (tb.end)
			{
				html += '<tbody class="highlight"><tr>';
				html += '<td rowspan="'+tbodySpan+'">'+HTMLHelper.colorMatch(aligned.specimen.patientName, null, 'Click to highlight this patient')+'</td>';
				html += '<td rowspan="'+tbodySpan+'">'+HTMLHelper.qiaCubeLink(rna.QIACUBE_DATE, rna.QIACUBE_RUN_NO, rna.QIACUBE_POSITION, qiacubefilter, false)+'</td>';
				if (libplatefilter)
				{
					html += '<td rowspan="'+tbodySpan+'">'+HTMLHelper.libPlateLink(libPlate.name, lib.bioWell.location, libplatefilter, false)+'</td>';
				}
				html += tbodyHtml;
				html += '</tbody>';
				tbodyHtml = '';
				tbodySpan = 0;
			}
		}
		
		Doc.element('itemTable').innerHTML = html;
		Doc.element('step-1-title').innerHTML = stepTitle + ' (' + numListed + ')';
		
		Doc.show('step-1');

		var cs = document.getElementsByClassName('case-summary');
		for (var i = 0; i < cs.length; i++)
		{
			Events.addEventHandler(cs[i], 'click', search.showCaseSummary);
		}
		
		var lnk = document.getElementsByClassName('qiacube-link');
		for (var i = 0; i < lnk.length; i++)
		{
			Events.addEventHandler(lnk[i], 'click', search.setQiaCubeFilter);
		}
		var lnk = document.getElementsByClassName('libplate-link');
		for (var i = 0; i < lnk.length; i++)
		{
			Events.addEventHandler(lnk[i], 'click', search.setLibPlateFilter);
		}
		
		var cm = document.getElementsByClassName('colormatch');
		for (var i = 0; i < cm.length; i++)
		{
			Events.addEventHandler(cm[i], 'click', search.colorMatch);
		}
	}
	
	
	/**
		Primary analysis of of problems related to alignments that
		can be determined without knowledge of context and without need
		to find information about related (same patient) alignments.
	*/
	search.primaryAnalysis = function()
	{
		// Reset this mapping, since we are only interested in alignments in the 
		// current context (eg. QiaCube run or LibPlate)
		alignmentsByPat = {};
		
		for (var itemNo = 0; itemNo < alignments.length; itemNo++)
		{
			var aligned = alignments[itemNo];
			
			// We need to be able to lookup alignments by name and patient name
			alignmentByName[aligned.name] = aligned;
			var patName = aligned.specimen.patientName;
			var alignmentsForPat = alignmentsByPat[patName] || [];
			if (alignmentsForPat.length == 0) alignmentsByPat[patName] = alignmentsForPat;
			alignmentsForPat[alignmentsForPat.length] = aligned;
			
			// Reset bug info
			aligned.bugged = 0;
			aligned.buglevel = 0;
			aligned.bugText = [];
			
			var cmp = aligned.compare;
			if (cmp.hasVerifiedMismatch)
			{
				aligned.bugged = 1;
				if (aligned.QC_GENOTYPE_VERIFIED)
				{
					// This is strange -- two VERIFIED alignments with a mismatch!!
					aligned.buglevel = 2;
					aligned.bugText[aligned.bugText.length] = 'Verified × 2 mismatch!!';
				}
				else
				{
					aligned.buglevel = 1;
					aligned.bugText[aligned.bugText.length] = 'Verified mismatch';
				}
			}
				
			if (aligned.QC_GENOTYPE_HET_PCT > 65 && !aligned.QC_GENOTYPE_VERIFIED)
			{
				aligned.bugged = 1;
				aligned.buglevel = 1;
				aligned.bugText[aligned.bugText.length] = 'High HET';
			}
		}
	}
	
	
	/**
		Analysis of problems related to a given context (eg. a QiaCube or LibPlate).
		The context is given by a all alignments matching the given filter.
		
		1. For each alignment in the context we will search for a LOW_MISMATCH warning.
		2. A LOW_MISMATCH means that we have two similar alignments that have different patients (very unlikely!)
		3. So, we take that other patient and search for other alignments in the given context that
		   have the same patient.
		4. If we find another alignment this could be a possible mixup with that position.
	*/
	search.contextAnalysis = function(filter, positionProperty)
	{
		for (var itemNo = 0; itemNo < alignments.length; itemNo++)
		{
			var aligned = alignments[itemNo];
			var filterValue = filter(aligned);
			if (filterValue == null) continue; // Not in the given context
			
			var cmp = aligned.compare;
			var possiblePositions = [];
			var possiblePats = [];
			
			for (var mNo = 0; mNo < cmp.messages.length; mNo++)
			{
				var msg = cmp.messages[mNo];
				if (msg.category != 'LOW_MISMATCH') continue; // We are only interested in 'LOW_MISMATCH' warnings
				
				var otherA = alignmentByName[msg.alignment];
				if (otherA.bugged) continue; // The other alignment is a known problem, so we ignore it here
				
				// If the other alignment is in the same context it may not be useful unless it is a verified alignment
				if (filter(otherA) && !otherA.QC_GENOTYPE_VERIFIED) continue;
				
				// This is our candidate patient
				var otherPat = otherA.specimen.patientName;
				
				// Find alignments in the same context for the candiate patient
				var alignmentsForPat = alignmentsByPat[otherPat] || [];
				for (var i = 0; i < alignmentsForPat.length; i++)
				{
					var otherB = alignmentsForPat[i];
					var posB = positionProperty(otherB);
					if (filter(otherB) && possiblePositions.indexOf(posB) == -1)
					{
						// Save the position (eg, Plate coordinate or QiaCube position)
						possiblePositions[possiblePositions.length] = posB;
					}
				}
				if (possiblePats.indexOf(otherPat) == -1) possiblePats[possiblePats.length] = otherPat;
			}
			
			if (possiblePositions.length > 0)
			{
				// If we have found other positions, we report them to the user
				aligned.bugged = 1;
				var text = 'Is this from ';
				for (var posNo = 0; posNo < possiblePositions.length; posNo++)
				{
					var pos = possiblePositions[posNo];
					if (posNo > 0) text += posNo == possiblePositions.length-1 ? ' or ' : ', ';
					text += HTMLHelper.colorMatch(filterValue + '@' + pos, '@'+pos, 'Click to highlight this position');
				}
				text += '/';
				for (var patNo = 0; patNo < possiblePats.length; patNo++)
				{
					var pat = possiblePats[patNo];
					if (patNo > 0) text += patNo == possiblePats.length-1 ? ' or ' : ', ';
					text += HTMLHelper.colorMatch(pat, null, 'Click to highlight this patient');
				}
				text += '?';
				aligned.bugText[aligned.bugText.length] = text;
			}
			else if (possiblePats.length > 0)
			{
				// We found a low mismatch but not any other sample with the same patient
				// The mixup is probably not in the selected context
				var text = 'Could not find sample related to ';
				for (var patNo = 0; patNo < possiblePats.length; patNo++)
				{
					var pat = possiblePats[patNo];
					if (patNo > 0) text += patNo == possiblePats.length-1 ? ' or ' : ', ';
					text += HTMLHelper.colorMatch(pat, null, 'Click to highlight this patient');
				}
				aligned.bugText[aligned.bugText.length] = text;
			}
		}
		
	}
	
	/**
		EVent handler for highlighting related items with the same
		colormatch tag. For all items tagged with 'colormatch' we highlight
		all elements with the same 'match' value as the clicked element.
		Two different colors are supported by holding down CTRL, ALT or SHIFT
		while clicking.
	*/
	search.colorMatch = function(event)
	{
		var toMatch = Data.get(event.currentTarget, 'match');
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		var matchedClass = specialKey ? 'colormatched-2' : 'colormatched';
		var cm = document.getElementsByClassName('colormatch');
		var add = !event.currentTarget.classList.contains(matchedClass);
		for (var i = 0; i < cm.length; i++)
		{
			var e = cm[i];
			Doc.addOrRemoveClass(e, matchedClass, Data.get(e, 'match') == toMatch && add);
		}
	}
	
	search.getAlignmentHtml = function(alignment, refAlignment)
	{
		var tmp = {};
		
		var markedName = refAlignment ? HTMLHelper.tagCommonPartOfName(refAlignment.name, alignment.name) : null;
		tmp.name = HTMLHelper.colorMatch(alignment.name, markedName, 'Click to highlight this alignment');
		
		var icons = '';
		var tooltip = '';
		
		icons += '<img src="../images/case_summary.png" class="link case-summary" data-name="'+Strings.encodeTags(alignment.name)+'" title="Show case summary">';
		if (alignment.compare.flagged)
		{
			icons += '<img src="../images/flag.png" title="This alignment is already flagged">';
		}
		if (alignment.QC_GENOTYPE_VERIFIED) 
		{
			var img = alignment.QC_GENOTYPE_VERIFIED == 'Lysate' ? 'gt-verified-lysate.png' : 'gt-verified.png';
			tooltip = Strings.encodeTags('Verified by '+alignment.QC_GENOTYPE_VERIFIED);
			icons += '<img src="../images/'+img+'" title="'+tooltip+'">';
		}
		if (alignment.bugged)
		{
			var img = bugIcons[alignment.buglevel];
			var imgTitle = alignment.buglevel > 0 && alignment.bugText.length > 0 ? alignment.bugText[0] : '';
			icons += '<img src="../images/'+img+'" title="'+imgTitle+'">';
		}
		if (alignment.bugText.length > 0)
		{
			tooltip = alignment.bugText.join('<br>');
		}
		
		tmp.icons = icons;
		tmp.tooltip = tooltip;
		return tmp;
	}
	
	search.getMessageHtml = function(message, aligned, alignedByName, qiacubefilter, libplatefilter)
	{
		var html = '';
		var msgClass= "bg-messages";
		if (message.alignment)
		{
			html += '<td class="dottedleft text-col">';
			var otherAligned = alignedByName[message.alignment];
			
			var tmp = search.getAlignmentHtml(otherAligned, aligned);
			html += tmp.name+tmp.icons;

			if (otherAligned)
			{
				if (otherAligned.buglevel == 1 && (qiacubefilter || libplatefilter)) msgClass += ' ignored-mismatch';
				html += '<td>'+Reggie.formatNumber(otherAligned.QC_GENOTYPE_HET_PCT, '%', -1)+'</td>';
				var rna = otherAligned.rna;
				var lib = otherAligned.lib;
				var libPlate = lib.bioWell.bioPlate;
				
				html += '<td>'+HTMLHelper.qiaCubeLink(rna.QIACUBE_DATE, rna.QIACUBE_RUN_NO, rna.QIACUBE_POSITION, qiacubefilter, false)+'</td>';
				html += '<td>'+HTMLHelper.libPlateLink(libPlate.name, lib.bioWell.location, libplatefilter, false)+'</td>';
				html += '<td>'+HTMLHelper.colorMatch(otherAligned.specimen.patientName, null, 'Click to highlight this patient')+'</td>';
			}
		}
		else 
		{
			html += '<td class="dottedleft" colspan="5"></td>';
		}
		html = '<td class="' + msgClass+ '" title="'+(message.messageTooltip || '')+'">'+Strings.encodeTags(message.message) + '</td>' + html;
		return html;
	}
	
	search.showCaseSummary = function(event)
	{
		var name = Data.get(event.currentTarget, 'name');
		var caseName = name.substring(0, 7);
		var url = '../reports/case_summary.jsp?ID='+App.getSessionId();
		url += '&caseName='+encodeURIComponent(caseName);
		url += '&pageType=popup';
		Dialogs.openPopup(url, 'CaseSummary'+caseName, 1000, 700);
	}
	

	return search;
}();

Doc.onLoad(Search.initPage);

/**
	Represents a QiaCube in the QiaCube filter list.
	The date and run number is the important information.
*/
function QiaCube(date, runNo)
{
	this.date = date;
	this.runNo = runNo;
	this.name = date + '#' + runNo;
	this.title = Reggie.reformatDate(date) + '#' + runNo;
}

/**
	Represents a LibPlate in the LibPlate filter list.
	The name is the only important information.
*/
function LibPlate(name)
{
	this.name = name;
}

// Placeholder for various sort functions
var Sort = function()
{
	var sort = {};
	
	// Sort items by name
	sort.sortByName = function(a, b)
	{
		return a.name == b.name ? 0 : (a.name < b.name ? -1 : 1);
	}

	// Sort alignments by patient name
	// If they are equals sort by alignment name
	sort.sortAlignmentsByPatientName = function(a, b)
	{
		var patA = a.specimen.patientName;
		var patB = b.specimen.patientName;
		return patA == patB ? sort.sortByName(a, b) : (patA < patB ? -1 : 1);
	}
	
	// Sort alignments by LibPlate position (columns before rows)
	sort.sortAlignmentsByLibPlatePosition = function(a, b)
	{
		var plateA = a.lib.bioWell.bioPlate.name;
		var plateB = b.lib.bioWell.bioPlate.name;

		if (plateA != plateB) return plateA < plateB ? -1 : 1;
		
		var posA = a.lib.bioWell;
		var posB = b.lib.bioWell;
		if (posA.column != posB.column) return posA.column < posB.column ? -1 : 1;
		
		return posA.row < posB.row ? -1 : 1;
	}
	
	// Sort alignments by QiaCube position
	sort.sortAlignmentsByQiaCubePosition = function(a, b)
	{
		var qiaCubeA = a.rna.QIACUBE_DATE + '#' + a.rna.QIACUBE_RUN_NO;
		var qiaCubeB = b.rna.QIACUBE_DATE + '#' + b.rna.QIACUBE_RUN_NO;

		if (qiaCubeA != qiaCubeB) return qiaCubeA < qiaCubeB ? -1 : 1;

		var posA = a.rna.QIACUBE_POSITION;
		var posB = b.rna.QIACUBE_POSITION;
		return posA < posB ? -1 : 1;
	}
	
	return sort;
}();

//Placeholder for various filter functions
var Filter = function()
{
	var filter = {};
	
	// Creates a filter that accepts alignments related to
	// the given QiaCube run. The filter value should
	// be formatted as <date>#<runNo>
	filter.qiaCubeFilter = function(qiaCubeRun)
	{
		return function(alignment)
		{
			var rna = alignment.rna;
			return qiaCubeRun == rna.QIACUBE_DATE + '#' + rna.QIACUBE_RUN_NO ? qiaCubeRun : null;
		}
	}
	
	// Creates a filter that accepts alignments related
	// to the given LibPlate. The filter value is the name
	// of the plate,
	filter.libPlateFilter = function(libPlateName)
	{
		return function(alignment)
		{
			return libPlateName == alignment.lib.bioWell.bioPlate.name ? libPlateName : null;
		}
	}
	
	return filter;
}();

// Placeholder for various functions that help us get a property from an alignment 
var Property = function()
{
	var property = {};
	
	// Get the QiaCube position from the alignment
	property.QiaCubePosition = function(alignment)
	{
		return alignment.rna.QIACUBE_POSITION;
	}
	
	// Get the LibPlate position from the alignment
	property.LibPlatePosition = function(alignment)
	{
		return alignment.lib.bioWell.location;
	}
	
	return property;
}();

/**
	Helper functions for generating various HTML outputs.
*/
var HTMLHelper = function()
{
	var helper = {};
	
	var colorMatchTargets = {};
	var colorMatchIndex = 0;
	
	/**
		Create the link for a QiaCube run. It will typically show the date#run@pos
		and will also output colormatch functionality for either the entire QiaCube run 
		or the position (in filtered view)
	*/
	helper.qiaCubeLink = function(qiaCubeDate, qiaCubeRun, qiaCubePos, filterValue, shortForm)
	{
		var runId = qiaCubeDate+'#'+qiaCubeRun;
		var runPosId = runId+'@'+qiaCubePos;
		var html = '';
		if (!shortForm)
		{
			html += '<span class="link qiacube-link" data-qiacuberun="'+runId+'" title="Filter on this QiaCube run">';
			html += Reggie.reformatDate(qiaCubeDate) + '#' + qiaCubeRun;
			html += '</span>';
		}
		html += '@'+qiaCubePos;
		var isFiltered = filterValue == runId;
		return HTMLHelper.colorMatch(isFiltered ? runPosId : runId, html, isFiltered ? 'Click to highlight this position' : 'Click to highlight this QiaCube');
	}
	
	/**
		Create the link for a LibPlate position. It will typically show the name and position
		and will also output colormatch functionality for either the entire LibPlate 
		or the position (in filtered view).
	*/
	helper.libPlateLink = function(libPlateName, libPlatePos, filterValue, shortForm)
	{
		var html = '';
		if (!shortForm)
		{
			html = '<span class="link libplate-link" data-libplate="'+libPlateName+'" title="Filter on this plate">';
			html += Strings.encodeTags(libPlateName);
			html += '</span>&nbsp;';
		}
		html += libPlatePos;
		var isFiltered = filterValue == libPlateName;
		return HTMLHelper.colorMatch(isFiltered ? libPlateName+'@'+libPlatePos : libPlateName, html, isFiltered ? 'Click to highlight this position' : 'Click to highlight this LibPlate');
	}
	
	/**
		Return 'name2' with markup indicating which parts of the name
		are common with 'name1'.
	*/
	helper.tagCommonPartOfName = function(name1, name2)
	{
		var n1 = name1.split('.');
		var n2 = name2.split('.');
		var i = 0;
		
		var common = '';
		var sep = '.';
		while (i < n1.length && i < n2.length && n1[i] == n2[i])
		{
			common += n1[i] + sep;
			i++;
		}
		
		var diff = '';
		sep = '';
		while (i < n2.length)
		{
			diff += sep + n2[i];
			i++;
			sep = '.';
		}
		
		var html = '';
		if (common.length > 0) html += '<span class="commonpart">'+Strings.encodeTags(common)+'</span>';
		if (diff.length > 0) html += '<span class="diffpart">'+Strings.encodeTags(diff)+'</span>';
		return html;
	}

	
	// Creates a "ColorMatch" tag for the given text
	helper.colorMatch = function(plainText, formattedText, tooltip)
	{
		var target = colorMatchTargets[plainText];
		if (!target)
		{
			colorMatchIndex++;
			target = 'CM'+colorMatchIndex;
			colorMatchTargets[plainText] = target;
		}
		if (!formattedText) formattedText = Strings.encodeTags(plainText);

		var html = '<span class="colormatch" data-match="'+target+'"';
		if (tooltip) html += ' title="'+tooltip+'"';
		html += '>'+formattedText+'</span>';
		return html;
	}
	
	return helper;
}();
